﻿//css_ref System.Data;
//css_ref System.Transactions;
//css_ref System.Xml;
//css_ref Interop.SAPbobsCOM.dll;
//css_ref log4net.dll;
//css_ref Produmex.Foundation.Data.dll;
//css_ref Produmex.Foundation.Data.Sbo.dll;
//css_ref Produmex.Foundation.dll;
//css_ref Produmex.Foundation.Sbo.dll;
//css_ref Produmex.Sbo.Logex.Data.dll;
//css_ref Produmex.Sbo.Logex.LenL.dll;

using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Resources;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.LenL.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.LenL.Generators.PickListRobot;
using System;
using System.Reflection;
using System.Transactions;
using Produmex.Sbo.Logex.LenL.BusinessObjects;

namespace AddUpdateData
{
	class Program
    {
        static string CONNECTION_STRING = "Server=localhost;CompanyDB=SBODemo_US_314;ExtraDb=PMX_SBODemo_US_314;UseTrusted=False;DbUserName=sa;DbPassword=B1Admin;UserName=manager;Password=produmex;Pooling=True;MinPoolSize=2;MaxPoolSize=10;ServerType=MSSQL2005;DBCultureInfo=en-US;LicenseServer=localhost:30000";


        /// <summary>
        /// Used for logging
        /// </summary>
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);


        static void Main(string[] args)
        {
            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (SboConnection conn = new SboConnection(CONNECTION_STRING))
				{
					conn.Open(); //Open connection to SAP
					Console.WriteLine("Connection is open to database");

					PmxDbVersionUpdaterProvider dbVersionProv = new PmxDbVersionUpdaterProvider(conn);

					AddUpdateExtensionParameter_UPS(dbVersionProv);
					AddUpdateExtensionParameters_PicklistRobot(dbVersionProv);

                    PmxDbVersionUpdaterProvider updateProv = new PmxDbVersionUpdaterProvider(conn);
                    //Pick list type
                    updateProv.AddUpdateUserDefinedTable(typeof(PmxPickListTypeFQ));
                    updateProv.AddUpdateUserDefinedObject(typeof(PmxPickListTypeFQ));

                    //Priority configuration
                    updateProv.AddUpdateUserDefinedTable(typeof(PmxPriorityConfigurationFQ));
                    updateProv.AddUpdateUserDefinedObject(typeof(PmxPriorityConfigurationFQ));
                }

				//Complete transaction
				scope.Complete();
            }


            using (TransactionScope scope = PmxDbConnection.GetNewTransactionScope())
            {
                using (SboConnection conn = new SboConnection(CONNECTION_STRING))
                {
                    conn.Open(); //Open connection to SAP
                    Console.WriteLine("Connection is open to database");

					RunTableOrViewSqlScript("PMX_LL_UPSN", conn, false);
					RunTableOrViewSqlScript("PMX_LNL_SALES_ORDERS_FOR_CREATING_PICKLISTS", conn, true);
					RunTableOrViewSqlScript("PMX_LNL_WHS_TRANSFER_REQUESTS_FOR_CREATING_PICKLISTS", conn, true);
					RunTableOrViewSqlScript("PMX_LNL_PICKLISTS_FOR_GROUPING_BY_PRIORITY", conn, true);
					RunTableOrViewSqlScript("PMX_LNL_PICKLISTS_FOR_GROUPING_WITHOUT_PRIORITY", conn, true);
				}

                //Complete transaction
                scope.Complete();
            }
        }

		private static void AddUpdateExtensionParameter_UPS(PmxDbVersionUpdaterProvider dbVersionProv)
		{
			dbVersionProv.AddUpdateExtensionParameter(PmxExtensionParameterCodesLenL.UPS_ACCOUNT_NUMBER,
				"L&L - UPS label Account Number",
				Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionCodes.PACKING_THINCLIENT_PARAMETERSET,
				PmxExtensionCodes.SYSTEM_STRING_CONVERTOR,
				"123456",
				PmxExtensionParameterCodesLenL.UPS_ACCOUNT_NUMBER.Substring(4));

			dbVersionProv.AddUpdateExtensionParameter(PmxExtensionParameterCodesLenL.UPS_SERVICE_CODE,
				"L&L - UPS label Service Indicator",
				Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionCodes.PACKING_THINCLIENT_PARAMETERSET,
				PmxExtensionCodes.SYSTEM_STRING_CONVERTOR,
				"78",
				PmxExtensionParameterCodesLenL.UPS_SERVICE_CODE.Substring(4));

			dbVersionProv.AddUpdateExtensionParameter(PmxExtensionParameterCodesLenL.UPS_FIRST_PTN,
				"L&L - UPS label eerste PTN",
				Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionCodes.PACKING_THINCLIENT_PARAMETERSET,
				PmxExtensionCodes.SYSTEM_INT32_CONVERTOR,
				"0",
				PmxExtensionParameterCodesLenL.UPS_FIRST_PTN.Substring(4));

			dbVersionProv.AddUpdateExtensionParameter(PmxExtensionParameterCodesLenL.UPS_LAST_PTN,
				"L&L - UPS label laatste PTN",
				Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionCodes.PACKING_THINCLIENT_PARAMETERSET,
				PmxExtensionCodes.SYSTEM_INT32_CONVERTOR,
				"9999999",
				PmxExtensionParameterCodesLenL.UPS_LAST_PTN.Substring(4));

			dbVersionProv.AddUpdateExtensionParameter(PmxExtensionParameterCodesLenL.UPS_NEXT_PTN,
				"L&L - UPS label volgende PTN",
				Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionCodes.PACKING_THINCLIENT_PARAMETERSET,
				PmxExtensionCodes.SYSTEM_INT32_CONVERTOR,
				"000000",
				PmxExtensionParameterCodesLenL.UPS_NEXT_PTN.Substring(4));

			dbVersionProv.AddUpdateExtensionParameter(PmxExtensionParameterCodesLenL.UPS_XML_OUT_DIR,
				"L&L - UPS label export folder",
				Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionCodes.PACKING_THINCLIENT_PARAMETERSET,
				PmxExtensionCodes.SYSTEM_STRING_CONVERTOR,
				@"C:\UPS\WSTD\ImpExp\Auto Import",
				PmxExtensionParameterCodesLenL.UPS_XML_OUT_DIR.Substring(4));
		}

		private static void AddUpdateExtensionParameters_PicklistRobot(PmxDbVersionUpdaterProvider dbVersionProv)
		{
			dbVersionProv.AddUpdateExtension(
				PickListRobot.IDs.ExtCode, 
				"FittinQ Pick List robot", 
				typeof(PickListRobot), 
				Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxExtensionTypeCodes.IPICKLISTROBOT);

			AddUpdateStringExtensionParameter_PickListRobot(dbVersionProv,
				PickListRobot.IDs.ExtParamViewSalesOrders,
				"View name of Sales Orders for creating Picklists",
				"PMX_LNL_SALES_ORDERS_FOR_CREATING_PICKLISTS",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_VIEW_SALES_ORDERS
				);

			AddUpdateStringExtensionParameter_PickListRobot(dbVersionProv,
				PickListRobot.IDs.ExtParamViewSalesOrdersOrderBy,
				"View OrderBy of Sales Orders for creating Picklists",
				"SalesOrderDocEntry",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_VIEW_SALES_ORDERS_ORDER_BY
				);

			AddUpdateStringExtensionParameter_PickListRobot(dbVersionProv,
				PickListRobot.IDs.ExtParamViewWhsTransferRequests,
				"View name of Whs Transfer Requests for creating Picklists",
				"PMX_LNL_WHS_TRANSFER_REQUESTS_FOR_CREATING_PICKLISTS",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_VIEW_WHS_TRANSFER_REQUESTS
				);

			AddUpdateStringExtensionParameter_PickListRobot(dbVersionProv,
				PickListRobot.IDs.ExtParamViewWhsTransferRequestsOrderBy,
				"View OrderBy of Whs Transfer Requests for creating Picklists",
				"WhsTransferRequestDocEntry",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_VIEW_WHS_TRANSFER_REQUESTS_ORDER_BY
				);

			AddUpdateStringExtensionParameter_PickListRobot(dbVersionProv,
				PickListRobot.IDs.ExtParamViewGroupPicklistsByPriority,
				"View name of Picklists for grouping by priority",
				"PMX_LNL_PICKLISTS_FOR_GROUPING_BY_PRIORITY",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_VIEW_GROUP_PICKLISTS_BY_PRIORITY
				);

			AddUpdateStringExtensionParameter_PickListRobot(dbVersionProv,
				PickListRobot.IDs.ExtParamViewGroupPicklistsByPriorityOrderBy,
				"View OrderBy of Picklists for grouping by priority",
				"PicklistDocEntry",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_VIEW_GROUP_PICKLISTS_BY_PRIORITY_ORDER_BY
				);

			AddUpdateStringExtensionParameter_PickListRobot(dbVersionProv,
				PickListRobot.IDs.ExtParamViewGroupPicklistsNoPriority,
				"View name of Picklists for grouping without priority",
				"PMX_LNL_PICKLISTS_FOR_GROUPING_WITHOUT_PRIORITY",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_VIEW_GROUP_PICKLISTS_NO_PRIORITY
				);

			AddUpdateStringExtensionParameter_PickListRobot(dbVersionProv,
				PickListRobot.IDs.ExtParamViewGroupPicklistsNoPriorityOrderBy,
				"View OrderBy of Picklists for grouping without priority",
				"PicklistDocEntry",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_VIEW_GROUP_PICKLISTS_NO_PRIORITY_ORDER_BY
				);

			dbVersionProv.AddUpdateExtensionParameter(PickListRobot.IDs.ExtParamPicklistTypeForWhsTransferRequests,
				"Picklist type for Whs Transfer Request Picklists",
				PickListRobot.IDs.ExtCode,
				PmxExtensionCodes.SYSTEM_STRING_VALID_VALUE_CONVERTOR,
				PmxPickListTypeColumnValues.Standard.VALUE,
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_PICKLIST_TYPE_FOR_WHS_TRANSFER_REQUEST_PICKLISTS
				);

			dbVersionProv.AddUpdateParameterValidValue(
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_PICKLIST_TYPE_FOR_WHS_TRANSFER_REQUEST_PICKLISTS,
				PmxPickListTypeColumnValues.Standard.VALUE, PmxPickListTypeColumnValues.Standard.DESCR);
			dbVersionProv.AddUpdateParameterValidValue(
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_PICKLIST_TYPE_FOR_WHS_TRANSFER_REQUEST_PICKLISTS,
				PmxPickListTypeColumnValues.CrossDocking.VALUE, PmxPickListTypeColumnValues.CrossDocking.DESCR);

			dbVersionProv.AddUpdateExtensionParameter(PickListRobot.IDs.ExtParamMaxNumberOfSpacesForWave,
				"Max number of spaces for wave",
				PickListRobot.IDs.ExtCode,
				PmxExtensionCodes.SYSTEM_INT32_CONVERTOR,
				"48",
				PmxExtensionParameterCodesLenL.PICKLIST_ROBOT_MAX_NUMBER_OF_SPACES_FOR_WAVE
				);
		}

		private static void AddUpdateStringExtensionParameter_PickListRobot(PmxDbVersionUpdaterProvider updateProv, string code, string name, string defaultValue, string generalParameterCode)
		{
			updateProv.AddUpdateExtensionParameter(code,
				name,
				PickListRobot.IDs.ExtCode,
				PmxExtensionCodes.SYSTEM_STRING_CONVERTOR,
				defaultValue,
				generalParameterCode
				);
		}

		private static void RunTableOrViewSqlScript(string tableOrViewName, PmxDbConnection conn, bool isView)
        {
			Console.WriteLine("Add" + (isView ? " view " : " table ") + tableOrViewName);
			string content = ResourceHelper.ReadResourceContentAsString(Assembly.GetAssembly(typeof(Produmex.Sbo.Logex.LenL.SboCommands)), "Produmex.Sbo.Logex.LenL.Scripts." + (isView ? "Views." : "Tables.") + tableOrViewName + ".sql");
            SboRecordsetHelper.RunSqlScript(s_log, content, conn);
        }
    }
}
